@@ -1,7 +1,7 @@
 /*
  * nilfs.c - NILFS library.
  *
- * Copyright (C) 2007-2010 Nippon Telegraph and Telephone Corporation.
+ * Copyright (C) 2007-2012 Nippon Telegraph and Telephone Corporation.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -17,9 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * Written by Koji Sato <koji@osrg.net>.
+ * Written by Koji Sato.
  *
- * Maintained by Ryusuke Konishi <ryusuke@osrg.net> from 2008.
+ * Maintained by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp> from 2008.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -73,13 +73,15 @@
 #include <errno.h>
 #include <assert.h>
 #include "nilfs.h"
+#include "util.h"
+#include "pathnames.h"
 #include "realpath.h"
 
 extern __u32 crc32_le(__u32 seed, unsigned char const *data, size_t length);
 
 static inline int iseol(int c)
 {
-	return (c == '\n' || c == '\0');
+	return c == '\n' || c == '\0';
 }
 
 static size_t tokenize(char *line, char **tokens, size_t ntoks)
@@ -123,11 +125,13 @@ static int has_mntopt(const char *opts, const char *opt)
 	p = opts;
 	len = strlen(opt);
 	while (p != NULL) {
-		if ((q = strchr(p, MNTOPT_SEP)) != NULL) {
-			n = (q - p < len) ? len : q - p;
+		q = strchr(p, MNTOPT_SEP);
+		if (q) {
+			n = max_t(size_t, q - p, len);
 			q++;
-		} else
-			n = (strlen(p) < len) ? len : strlen(p);
+		} else {
+			n = max_t(size_t, strlen(p), len);
+		}
 		if (strncmp(p, opt, n) == 0)
 			return 1;
 		p = q;
@@ -138,7 +142,6 @@ static int has_mntopt(const char *opts, const char *opt)
 #ifndef LINE_MAX
 #define LINE_MAX	2048
 #endif	/* LINE_MAX */
-#define PROCMOUNTS	"/proc/mounts"
 
 static int nilfs_find_fs(struct nilfs *nilfs, const char *dev, const char *dir,
 			 const char *opt)
@@ -165,7 +168,7 @@ static int nilfs_find_fs(struct nilfs *nilfs, const char *dev, const char *dir,
 		dir = cdir;
 	}
 
-	fp = fopen(PROCMOUNTS, "r");
+	fp = fopen(_PATH_PROC_MOUNTS, "r");
 	if (fp == NULL)
 		goto failed_dir;
 
@@ -222,12 +225,20 @@ static int nilfs_find_fs(struct nilfs *nilfs, const char *dev, const char *dir,
 	return ret;
 }
 
+/**
+ * nilfs_get_block_size - get block size of the file system
+ * @nilfs: nilfs object
+ */
 size_t nilfs_get_block_size(struct nilfs *nilfs)
 {
 	return 1UL << (le32_to_cpu(nilfs->n_sb->s_log_block_size) +
 		       NILFS_SB_BLOCK_SIZE_SHIFT);
 }
 
+/**
+ * nilfs_get_reserved_segments - get number of reserved segments
+ * @nilfs: nilfs object
+ */
 __u64 nilfs_get_reserved_segments(const struct nilfs *nilfs)
 {
 	const struct nilfs_super_block *sb = nilfs->n_sb;
@@ -240,12 +251,17 @@ __u64 nilfs_get_reserved_segments(const struct nilfs *nilfs)
 	return rn;
 }
 
+/**
+ * nilfs_opt_set_mmap - set mmap option
+ * @nilfs: nilfs object
+ */
 int nilfs_opt_set_mmap(struct nilfs *nilfs)
 {
 	long pagesize;
 	size_t segsize;
 
-	if ((pagesize = sysconf(_SC_PAGESIZE)) < 0)
+	pagesize = sysconf(_SC_PAGESIZE);
+	if (pagesize < 0)
 		return -1;
 	segsize = le32_to_cpu(nilfs->n_sb->s_blocks_per_segment) *
 		nilfs_get_block_size(nilfs);
@@ -256,16 +272,52 @@ int nilfs_opt_set_mmap(struct nilfs *nilfs)
 	return 0;
 }
 
+/**
+ * nilfs_opt_clear_mmap - clear mmap option
+ * @nilfs: nilfs object
+ */
 void nilfs_opt_clear_mmap(struct nilfs *nilfs)
 {
 	nilfs->n_opts &= ~NILFS_OPT_MMAP;
 }
 
+/**
+ * nilfs_opt_test_mmap - test whether mmap option is set or not
+ * @nilfs: nilfs object
+ */
 int nilfs_opt_test_mmap(struct nilfs *nilfs)
 {
 	return !!(nilfs->n_opts & NILFS_OPT_MMAP);
 }
 
+/**
+ * nilfs_opt_set_set_suinfo - set set_suinfo option
+ * @nilfs: nilfs object
+ */
+int nilfs_opt_set_set_suinfo(struct nilfs *nilfs)
+{
+	nilfs->n_opts |= NILFS_OPT_SET_SUINFO;
+	return 0;
+}
+
+/**
+ * nilfs_opt_clear_set_suinfo - clear set_suinfo option
+ * @nilfs: nilfs object
+ */
+void nilfs_opt_clear_set_suinfo(struct nilfs *nilfs)
+{
+	nilfs->n_opts &= ~NILFS_OPT_SET_SUINFO;
+}
+
+/**
+ * nilfs_opt_test_set_suinfo - test whether set_suinfo option is set or not
+ * @nilfs: nilfs object
+ */
+int nilfs_opt_test_set_suinfo(struct nilfs *nilfs)
+{
+	return !!(nilfs->n_opts & NILFS_OPT_SET_SUINFO);
+}
+
 static int nilfs_open_sem(struct nilfs *nilfs)
 {
 	char semnambuf[NAME_MAX - 4];
@@ -329,6 +381,7 @@ struct nilfs *nilfs_open(const char *dev, const char *dir, int flags)
 	nilfs->n_iocfd = -1;
 	nilfs->n_dev = NULL;
 	nilfs->n_ioc = NULL;
+	nilfs->n_opts = 0;
 	nilfs->n_mincno = NILFS_CNO_MIN;
 	memset(nilfs->n_sems, 0, sizeof(nilfs->n_sems));
 
@@ -359,9 +412,9 @@ struct nilfs *nilfs_open(const char *dev, const char *dir, int flags)
 	    (NILFS_OPEN_RDONLY | NILFS_OPEN_WRONLY | NILFS_OPEN_RDWR)) {
 		if (nilfs_find_fs(nilfs, dev, dir, MNTOPT_RW) < 0) {
 			if (!(flags & NILFS_OPEN_RDONLY))
-				goto out_nilfs;
+				goto out_fd;
 			if (nilfs_find_fs(nilfs, dev, dir, MNTOPT_RO) < 0)
-				goto out_nilfs;
+				goto out_fd;
 		}
 		nilfs->n_iocfd = open(nilfs->n_ioc, O_RDONLY);
 		if (nilfs->n_iocfd < 0)
@@ -383,14 +436,10 @@ out_fd:
 		close(nilfs->n_devfd);
 	if (nilfs->n_iocfd >= 0)
 		close(nilfs->n_iocfd);
-	if (nilfs->n_dev != NULL)
-		free(nilfs->n_dev);
-	if (nilfs->n_ioc != NULL)
-		free(nilfs->n_ioc);
-	if (nilfs->n_sb != NULL)
-		free(nilfs->n_sb);
 
-out_nilfs:
+	free(nilfs->n_dev);
+	free(nilfs->n_ioc);
+	free(nilfs->n_sb);
 	free(nilfs);
 	return NULL;
 }
@@ -407,18 +456,16 @@ void nilfs_close(struct nilfs *nilfs)
 		close(nilfs->n_devfd);
 	if (nilfs->n_iocfd >= 0)
 		close(nilfs->n_iocfd);
-	if (nilfs->n_dev != NULL)
-		free(nilfs->n_dev);
-	if (nilfs->n_ioc != NULL)
-		free(nilfs->n_ioc);
-	if (nilfs->n_sb != NULL)
-		free(nilfs->n_sb);
+
+	free(nilfs->n_dev);
+	free(nilfs->n_ioc);
+	free(nilfs->n_sb);
 	free(nilfs);
 }
 
 /**
- * nilfs_get_dev -
- * @nilfs:
+ * nilfs_get_dev - get the name of a device that nilfs object is opening
+ * @nilfs: nilfs object
  */
 const char *nilfs_get_dev(const struct nilfs *nilfs)
 {
@@ -426,10 +473,10 @@ const char *nilfs_get_dev(const struct nilfs *nilfs)
 }
 
 /**
- * nilfs_change_cpmode -
- * @nilfs:
- * @cno:
- * @mode:
+ * nilfs_change_cpmode - change mode of a checkpoint
+ * @nilfs: nilfs object
+ * @cno: checkpoint number
+ * @mode: new mode of checkpoint
  */
 int nilfs_change_cpmode(struct nilfs *nilfs, nilfs_cno_t cno, int mode)
 {
@@ -446,16 +493,17 @@ int nilfs_change_cpmode(struct nilfs *nilfs, nilfs_cno_t cno, int mode)
 
 	cpmode.cm_cno = cno;
 	cpmode.cm_mode = mode;
+	cpmode.cm_pad = 0;
 	return ioctl(nilfs->n_iocfd, NILFS_IOCTL_CHANGE_CPMODE, &cpmode);
 }
 
 /**
- * nilfs_get_cpinfo -
- * @nilfs:
- * @cno:
- * @mode:
- * @cpinfo:
- * @nci:
+ * nilfs_get_cpinfo - get information of checkpoints
+ * @nilfs: nilfs object
+ * @cno: start checkpoint number
+ * @mode: mode of checkpoints that the caller wants to retrieve
+ * @cpinfo: array of nilfs_cpinfo structs to store information in
+ * @nci: size of @cpinfo array (number of items)
  */
 ssize_t nilfs_get_cpinfo(struct nilfs *nilfs, nilfs_cno_t cno, int mode,
 			 struct nilfs_cpinfo *cpinfo, size_t nci)
@@ -490,9 +538,9 @@ ssize_t nilfs_get_cpinfo(struct nilfs *nilfs, nilfs_cno_t cno, int mode,
 }
 
 /**
- * nilfs_delete_checkpoint -
- * @nilfs:
- * @cno:
+ * nilfs_delete_checkpoint - delete a checkpoint
+ * @nilfs: nilfs object
+ * @cno: checkpoint number to be deleted
  */
 int nilfs_delete_checkpoint(struct nilfs *nilfs, nilfs_cno_t cno)
 {
@@ -504,9 +552,9 @@ int nilfs_delete_checkpoint(struct nilfs *nilfs, nilfs_cno_t cno)
 }
 
 /**
- * nilfs_get_cpstat -
- * @nilfs:
- * @cpstat:
+ * nilfs_get_cpstat - get checkpoint statistics
+ * @nilfs: nilfs object
+ * @cpstat: buffer of a nilfs_cpstat struct to store statistics in
  */
 int nilfs_get_cpstat(const struct nilfs *nilfs, struct nilfs_cpstat *cpstat)
 {
@@ -518,11 +566,11 @@ int nilfs_get_cpstat(const struct nilfs *nilfs, struct nilfs_cpstat *cpstat)
 }
 
 /**
- * nilfs_get_suinfo -
- * @nilfs:
- * @segnum:
- * @si:
- * @nsi:
+ * nilfs_get_suinfo - get information of segment usage
+ * @nilfs: nilfs object
+ * @segnum: start segment number
+ * @si: array of nilfs_suinfo structs to store information in
+ * @nsi: size of @si array (number of items)
  */
 ssize_t nilfs_get_suinfo(const struct nilfs *nilfs, __u64 segnum,
 			 struct nilfs_suinfo *si, size_t nsi)
@@ -537,6 +585,7 @@ ssize_t nilfs_get_suinfo(const struct nilfs *nilfs, __u64 segnum,
 	argv.v_base = (unsigned long)si;
 	argv.v_nmembs = nsi;
 	argv.v_size = sizeof(struct nilfs_suinfo);
+	argv.v_flags = 0;
 	argv.v_index = segnum;
 	if (ioctl(nilfs->n_iocfd, NILFS_IOCTL_GET_SUINFO, &argv) < 0)
 		return -1;
@@ -544,9 +593,40 @@ ssize_t nilfs_get_suinfo(const struct nilfs *nilfs, __u64 segnum,
 }
 
 /**
- * nilfs_get_sustat -
- * @nilfs:
- * @sustat:
+ * nilfs_set_suinfo - sets segment usage info
+ * @nilfs: nilfs object
+ * @sup: an array of nilfs_suinfo_update structs
+ * @nsup: number of elements in sup
+ *
+ * Description: Takes an array of nilfs_suinfo_update structs and updates
+ * segment usage info accordingly. Only the fields indicated by sup_flags
+ * are updated.
+ *
+ * Return Value: On success, 0 is returned. On error, -1 is returned.
+ */
+int nilfs_set_suinfo(const struct nilfs *nilfs,
+		     struct nilfs_suinfo_update *sup, size_t nsup)
+{
+	struct nilfs_argv argv;
+
+	if (nilfs->n_iocfd < 0) {
+		errno = EBADF;
+		return -1;
+	}
+
+	argv.v_base = (unsigned long)sup;
+	argv.v_nmembs = nsup;
+	argv.v_size = sizeof(struct nilfs_suinfo_update);
+	argv.v_index = 0;
+	argv.v_flags = 0;
+
+	return ioctl(nilfs->n_iocfd, NILFS_IOCTL_SET_SUINFO, &argv);
+}
+
+/**
+ * nilfs_get_sustat - get segment usage statistics
+ * @nilfs: nilfs object
+ * @sustat: buffer of a nilfs_sustat struct to store statistics in
  */
 int nilfs_get_sustat(const struct nilfs *nilfs, struct nilfs_sustat *sustat)
 {
@@ -559,10 +639,10 @@ int nilfs_get_sustat(const struct nilfs *nilfs, struct nilfs_sustat *sustat)
 }
 
 /**
- * nilfs_get_vinfo -
- * @nilfs:
- * @vinfo:
- * @nvi:
+ * nilfs_get_vinfo - get information of virtual block addresses
+ * @nilfs: nilfs object
+ * @vinfo: array of nilfs_vinfo structs to store information in
+ * @nvi: size of @vinfo array (number of items)
  */
 ssize_t nilfs_get_vinfo(const struct nilfs *nilfs,
 			struct nilfs_vinfo *vinfo, size_t nvi)
@@ -577,16 +657,18 @@ ssize_t nilfs_get_vinfo(const struct nilfs *nilfs,
 	argv.v_base = (unsigned long)vinfo;
 	argv.v_nmembs = nvi;
 	argv.v_size = sizeof(struct nilfs_vinfo);
+	argv.v_flags = 0;
+	argv.v_index = 0;
 	if (ioctl(nilfs->n_iocfd, NILFS_IOCTL_GET_VINFO, &argv) < 0)
 		return -1;
 	return argv.v_nmembs;
 }
 
 /**
- * nilfs_get_bdescs:
- * @nilfs:
- * @bdescs:
- * @nbdescs:
+ * nilfs_get_bdescs - get information of blocks used in DAT file itself
+ * @nilfs: nilfs object
+ * @bdescs: array of nilfs_bdesc structs to store information in
+ * @nbdescs: size of @bdescs array (number of items)
  */
 ssize_t nilfs_get_bdescs(const struct nilfs *nilfs,
 			 struct nilfs_bdesc *bdescs, size_t nbdescs)
@@ -601,24 +683,26 @@ ssize_t nilfs_get_bdescs(const struct nilfs *nilfs,
 	argv.v_base = (unsigned long)bdescs;
 	argv.v_nmembs = nbdescs;
 	argv.v_size = sizeof(struct nilfs_bdesc);
+	argv.v_flags = 0;
+	argv.v_index = 0;
 	if (ioctl(nilfs->n_iocfd, NILFS_IOCTL_GET_BDESCS, &argv) < 0)
 		return -1;
 	return argv.v_nmembs;
 }
 
 /**
- * nilfs_clean_segments -
- * @nilfs:
- * @vdescs:
- * @nvdescs:
- * @periods:
- * @nperiods:
- * @vblocknrs:
- * @nvblocknrs:
- * @bdescs:
- * @nbdescs:
- * @segnums:
- * @nsegs:
+ * nilfs_clean_segments - do garbage collection operation
+ * @nilfs: nilfs object
+ * @vdescs: array of nilfs_vdesc structs to specify live blocks
+ * @nvdescs: size of @vdescs array (number of items)
+ * @periods: array of nilfs_period structs to specify checkpoints to be deleted
+ * @nperiods: size of @periods array (number of items)
+ * @vblocknrs: array of virtual block addresses to be freed
+ * @nvblocknrs: size of @vblocknrs array (number of items)
+ * @bdescs: array of nilfs_bdesc structs to specify live DAT blocks
+ * @nbdescs: size of @bdescs array (number of items)
+ * @segnums: array of segment numbers to specify segments to be freed
+ * @nsegs: size of @segnums array (number of items)
  */
 int nilfs_clean_segments(struct nilfs *nilfs,
 			 struct nilfs_vdesc *vdescs, size_t nvdescs,
@@ -634,6 +718,7 @@ int nilfs_clean_segments(struct nilfs *nilfs,
 		return -1;
 	}
 
+	memset(argv, 0, sizeof(argv));
 	argv[0].v_base = (unsigned long)vdescs;
 	argv[0].v_nmembs = nvdescs;
 	argv[0].v_size = sizeof(struct nilfs_vdesc);
@@ -653,24 +738,18 @@ int nilfs_clean_segments(struct nilfs *nilfs,
 }
 
 /**
- * nilfs_sync -
- * @nilfs:
+ * nilfs_sync - sync a NILFS file system
+ * @nilfs: nilfs object
+ * @cnop: buffer to store the latest checkpoint number in
  */
 int nilfs_sync(const struct nilfs *nilfs, nilfs_cno_t *cnop)
 {
-	int ret;
-
 	if (nilfs->n_iocfd < 0) {
 		errno = EBADF;
 		return -1;
 	}
 
-	if (((ret = ioctl(nilfs->n_iocfd, NILFS_IOCTL_SYNC, cnop)) < 0) &&
-	    (ret == -EROFS))
-		/* syncing read-only filesystem */
-		ret = 0;
-
-	return ret;
+	return ioctl(nilfs->n_iocfd, NILFS_IOCTL_SYNC, cnop);
 }
 
 /**
@@ -711,9 +790,10 @@ int nilfs_set_alloc_range(struct nilfs *nilfs, off_t start, off_t end)
 /* raw */
 
 /**
- * nilfs_get_segment -
- * @nilfs: NILFS object
+ * nilfs_get_segment - read or mmap segment to a memory region
+ * @nilfs: nilfs object
  * @segnum: segment number
+ * @segmentp: buffer to store start address of allocated memory region in
  */
 ssize_t nilfs_get_segment(struct nilfs *nilfs, unsigned long segnum,
 			  void **segmentp)
@@ -739,12 +819,14 @@ ssize_t nilfs_get_segment(struct nilfs *nilfs, unsigned long segnum,
 
 #ifdef HAVE_MMAP
 	if (nilfs_opt_test_mmap(nilfs)) {
-		if ((segment = mmap(0, segsize, PROT_READ, MAP_SHARED,
-				    nilfs->n_devfd, offset)) == MAP_FAILED)
+		segment = mmap(0, segsize, PROT_READ, MAP_SHARED,
+			       nilfs->n_devfd, offset);
+		if (segment == MAP_FAILED)
 			return -1;
 	} else {
 #endif	/* HAVE_MMAP */
-		if ((segment = malloc(segsize)) == NULL)
+		segment = malloc(segsize);
+		if (!segment)
 			return -1;
 		if (lseek(nilfs->n_devfd, offset, SEEK_SET) != offset) {
 			free(segment);
@@ -763,9 +845,9 @@ ssize_t nilfs_get_segment(struct nilfs *nilfs, unsigned long segnum,
 }
 
 /**
- * nilfs_put_segment -
- * @nilfs: NILFS object
- * @seg: segment
+ * nilfs_put_segment - free memory used for raw segment access
+ * @nilfs: nilfs object
+ * @segment: start address of the memory region to be freed
  */
 int nilfs_put_segment(struct nilfs *nilfs, void *segment)
 {
@@ -999,7 +1081,7 @@ void nilfs_block_next(struct nilfs_block *blk)
 nilfs_cno_t nilfs_get_oldest_cno(struct nilfs *nilfs)
 {
 	struct nilfs_cpinfo cpinfo[1];
-	
+
 	nilfs_get_cpinfo(nilfs, nilfs->n_mincno, NILFS_CHECKPOINT, cpinfo, 1);
 	return nilfs->n_mincno;
 }
